"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const list_1 = require("cli-ux/lib/list");
const cli_flags_1 = require("cli-flags");
const chalk_1 = require("chalk");
function buildUsage(command) {
    if (command.usage)
        return command.usage.trim();
    let cmd = command.id;
    if (!command.args)
        return (cmd || '').trim();
    let args = command.args.map(renderArg);
    return `${cmd} ${args.join(' ')}`.trim();
}
function renderArg(arg) {
    let name = arg.name.toUpperCase();
    if (arg.required)
        return `${name}`;
    else
        return `[${name}]`;
}
class Help {
    constructor(config) {
        this.config = config;
    }
    command(cmd) {
        let flags = Object.entries(cmd.flags || {})
            .filter(([, f]) => !f.hidden)
            .map(([k, f]) => {
            f.name = k;
            return f;
        });
        let args = (cmd.args || []).filter(a => !a.hidden);
        let hasFlags = flags.length ? ` ${chalk_1.default.blue('[flags]')}` : '';
        let usage = `${chalk_1.default.bold('Usage:')} ${this.config.bin} ${buildUsage(cmd)}${hasFlags}\n`;
        return [
            usage,
            cmd.description ? `\n${chalk_1.default.bold(cmd.description.trim())}\n` : '',
            this.renderAliases(this.config.aliases[cmd.id]),
            this.renderArgs(args),
            this.renderFlags(flags),
            cmd.help ? `\n${cmd.help.trim()}\n` : '',
        ].join('');
    }
    commandLine(cmd) {
        return [buildUsage(cmd), cmd.description ? chalk_1.default.dim(cmd.description) : undefined];
    }
    renderAliases(aliases) {
        if (!aliases || !aliases.length)
            return '';
        let a = aliases.map(a => `  $ ${this.config.bin} ${a}`).join('\n');
        return `\n${chalk_1.default.blue('Aliases:')}\n${a}\n`;
    }
    renderArgs(args) {
        if (!args.find(f => !!f.description))
            return '';
        return ('\n' +
            list_1.renderList(args.map(a => {
                return [a.name.toUpperCase(), a.description ? chalk_1.default.dim(a.description) : undefined];
            })) +
            '\n');
    }
    renderFlags(flags) {
        if (!flags.length)
            return '';
        return `\n${chalk_1.default.blue('Flags:')}\n` + list_1.renderList(cli_flags_1.flagUsages(flags)) + '\n';
    }
}
exports.Help = Help;
